/*;  khrbtree.cc {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl khrbtree.cc\quad\thepage}} */
/*;  $$Date: 1997/09/14 09:56:05 $$  */
/*;  $$Source: /home/shiiba/KhC++LibDevelop/trees/khrbtree.cc,v $$  */

/*
ΥץϡScott Robert Ladd ˤ C++ Templates and Tools
˷ǺܤƤץ򸵤ˤƤޤ

Υץϡ㳰(try, throw, catch ʤɤȤ)Υɤ
Ǥޤ̾Υ顼(顼å񤤤ơexit )
֤Ƥޤ

*/

#include "khrbtree.h"
#include "khboolean.h"
#include <assert.h>

template <class D>
void KhRedBlackTree<D>::operator =
(
 const KhRedBlackTree<D> & tree
)
{
  KhBinaryTree< KhRBData<D> >::operator = (tree);
}

template <class D>
void KhRedBlackTree<D>::RotateLeft
(
 KhTreeNode< KhRBData<D> > * node
)
{
  KhTreeNode< KhRBData<D> > * y = node->More;

  // turn y's left subtree into node's right subtree
  node->More = y->Less;

  if (y->Less != Sentinel)
    y->Less->Parent = node;

  // link node's parent to y
  y->Parent = node->Parent;

  if (node->Parent == Sentinel)
    Root = y;
  else
  {
    if (node == node->Parent->Less)
      node->Parent->Less = y;
    else
      node->Parent->More = y;
  }

  // put node on y's left
  y->Less = node;
  node->Parent = y;
}

template <class D>
void KhRedBlackTree<D>::RotateRight
(
 KhTreeNode< KhRBData<D> > * node
)
{
  KhTreeNode< KhRBData<D> > * y = node->Less;

  // turn y's right subtree into node's left subtree
  node->Less = y->More;

  if (y->More != Sentinel)
    y->More->Parent = node;

  // link node's parent to y
  y->Parent = node->Parent;

  if (node->Parent == Sentinel)
    Root = y;
  else
  {
    if (node == node->Parent->More)
      node->Parent->More = y;
    else
      node->Parent->Less = y;
  }

  // put node on y's right
  y->More = node;
  node->Parent = y;
}

template <class D>
void KhRedBlackTree<D>::DeleteFixup
(
 KhTreeNode< KhRBData<D> > * node
)
{
  KhTreeNode< KhRBData<D> > * w, * x = node;

  while ((x != Root) && (x->Data.Color == BLACK))
  {
    if (x == x->Parent->Less)
    {
      w = x->Parent->More;

      if (w->Data.Color == RED)
      {
	w->Data.Color = BLACK;
	x->Parent->Data.Color = RED;
	RotateLeft(x->Parent);
	w = x->Parent->More;
      }

      if ((w->Less->Data.Color == BLACK)
	  &&  (w->More->Data.Color == BLACK))
      {
	w->Data.Color = RED;
	x = x->Parent;
      }
      else
      {
	if (w->More->Data.Color == BLACK)
	{
	  w->Less->Data.Color = BLACK;
	  w->Data.Color = RED;
	  RotateRight(w);
	  w = x->Parent->More;
	}

	w->Data.Color = x->Parent->Data.Color;
	x->Parent->Data.Color = BLACK;
	w->More->Data.Color = BLACK;
	RotateLeft(x->Parent);
	x = Root;
      }
    }
    else
    {
      w = x->Parent->Less;

      if (w->Data.Color == RED)
      {
	w->Data.Color = BLACK;
	x->Parent->Data.Color = RED;
	RotateRight(x->Parent);
	w = x->Parent->Less;
      }

      if ((w->More->Data.Color == BLACK)
	  &&  (w->Less->Data.Color == BLACK))
      {
	w->Data.Color = RED;
	x = x->Parent;
      }
      else
      {
	if (w->Less->Data.Color == BLACK)
	{
	  w->More->Data.Color = BLACK;
	  w->Data.Color = RED;
	  RotateLeft(w);
	  w = x->Parent->Less;
	}

	w->Data.Color = x->Parent->Data.Color;
	x->Parent->Data.Color = BLACK;
	w->Less->Data.Color = BLACK;
	RotateRight(x->Parent);
	x = Root;
      }
    }
  }

  x->Data.Color = BLACK;
}

template <class D>
void KhRedBlackTree<D>::Insert
(
 const D & item
)
{
  KhRBData<D> rbitem(item);

  KhTreeNode< KhRBData<D> > * x = InternalInsert(rbitem);

  if (x == NULL) // replaced existing data; no new node
  return;

  KhTreeNode< KhRBData<D> > * y;

  x->Data.Color = RED;

  while ((x != Root) && (x->Parent->Data.Color == RED))
  {
    if (x->Parent == x->Parent->Parent->Less)
    {
      y = x->Parent->Parent->More;

      if (y->Data.Color == RED)
      {
	x->Parent->Data.Color = BLACK;
	y->Data.Color = BLACK;
	x->Parent->Parent->Data.Color = RED;
	x = x->Parent->Parent;
      }
      else
      {
	if (x == x->Parent->More)
	{
	  x = x->Parent;
	  RotateLeft(x);
	}

	x->Parent->Data.Color = BLACK;
	x->Parent->Parent->Data.Color = RED;
	RotateRight(x->Parent->Parent);
      }
    }
    else
    {
      y = x->Parent->Parent->Less;

      if (y->Data.Color == RED)
      {
	x->Parent->Data.Color = BLACK;
	y->Data.Color = BLACK;
	x->Parent->Parent->Data.Color = RED;
	x = x->Parent->Parent;
      }
      else
      {
	if (x == x->Parent->Less)
	{
	  x = x->Parent;
	  RotateRight(x);
	}

	x->Parent->Data.Color = BLACK;
	x->Parent->Parent->Data.Color = RED;
	RotateLeft(x->Parent->Parent);
      }
    }
  }

  Root->Data.Color = BLACK;
}

template <class D>
KhBoolean KhRedBlackTree<D>::Delete
(
 const D & item
)
{
  assert(LockCount <= 0);

  // find node
  KhTreeNode< KhRBData<D> > * z = Search(item);

  if (z == Sentinel)
  return NO;

  KhTreeNode< KhRBData<D> > * y, * x;

  // find node to splice out
  if ((z->Less == Sentinel) || (z->More == Sentinel))
    y = z;
  else
    y = Successor(z);

  // find child with which to replace y
  if (y->Less != Sentinel)
    x = y->Less;
  else
    x = y->More;

  // splice child onto parent
  x->Parent = y->Parent;

  if (y->Parent == Sentinel)
    Root = x; // replace root
  else
  {
    // splice in child node
    if (y == y->Parent->Less)
      y->Parent->Less = x;
    else
      y->Parent->More = x;
  }

  // if needed, save y data
  if (y != z)
  z->Data = y->Data;

  // adjust tree under red-black rules
  if (y->Data.Color == BLACK)
    DeleteFixup(x);

  // free memory
  DeleteNode(y);

  return YES;
}

template <class D>
D * KhRedBlackTree<D>::GetPtrToData
(
 const D & item
)
{
  KhTreeNode< KhRBData<D> >* x = Search(item);
  if (x == Sentinel)
    return NULL;
  else
    return &(x->Data.RealData);
}


template <class D>
KhRBTreeIterator<D>::KhRBTreeIterator
(
 KhRedBlackTree<D> & bt
)
  : KhBinaryTreeIterator< KhRBData<D> >(bt)
{
  // placeholder
}

// modified by Yutaka ICHIKAWA (Tue Jul 13 21:04:37 1999)
template <class D>
KhRBTreeIterator<D>::KhRBTreeIterator
(
 KhRBTreeIterator<D> & iter
)
  : KhBinaryTreeIterator< KhRBData<D> >(iter)
//   : BinaryTreeIterator< KhRBData<D> >(iter)
{
  // placeholder
}

// modified by Yutaka ICHIKAWA (Tue Jul 13 21:04:37 1999)
template <class D>
void KhRBTreeIterator<D>::operator =
(
 KhRBTreeIterator<D> & iter
)
{
//   BinaryTreeIterator< KhRBData<D> >::operator = (iter);
  KhBinaryTreeIterator< KhRBData<D> >::operator = (iter);
}

template <class D>
D KhRBTreeIterator<D>::operator * ()
{
//  assert(Node != Tree.GetSentinel());
  return Node->Data.RealData;
}

template <class D>
KhRBTreeIterator<D>::operator D* ()
{
  if (Node == Tree.GetSentinel())
    return NULL;
  else
    return &(Node->Data.RealData);
}

/*; end of file */
